@@ -110,7 +110,10 @@ def stage(
 
 @app.command("sdd")
 @beartype
-@require(lambda bundle: isinstance(bundle, str) and len(bundle) > 0, "Bundle name must be non-empty string")
+@require(
+    lambda bundle: bundle is None or (isinstance(bundle, str) and len(bundle) > 0),
+    "Bundle name must be None or non-empty string",
+)
 @require(lambda sdd: sdd is None or isinstance(sdd, Path), "SDD must be None or Path")
 @require(
     lambda output_format: isinstance(output_format, str) and output_format.lower() in ("yaml", "json", "markdown"),
@@ -168,7 +171,6 @@ def enforce_sdd(
     from rich.console import Console
 
     from specfact_cli.models.sdd import SDDManifest
-    from specfact_cli.utils.bundle_loader import load_project_bundle
     from specfact_cli.utils.structure import SpecFactStructure
     from specfact_cli.utils.structured_io import StructuredFormat
 
@@ -226,22 +228,11 @@ def enforce_sdd(
             sdd_manifest = SDDManifest.model_validate(sdd_data)
 
             # Load project bundle with progress indicator
-            from rich.progress import Progress, SpinnerColumn, TextColumn
 
-            with Progress(
-                SpinnerColumn(),
-                TextColumn("[progress.description]{task.description}"),
-                console=console,
-            ) as progress:
-                task = progress.add_task("Loading project bundle...", total=None)
+            from specfact_cli.utils.progress import load_bundle_with_progress
 
-                def progress_callback(current: int, total: int, artifact: str) -> None:
-                    progress.update(task, description=f"Loading artifact {current}/{total}: {artifact}")
-
-                project_bundle = load_project_bundle(
-                    bundle_dir, validate_hashes=False, progress_callback=progress_callback
-                )
-                progress.update(task, description="✓ Bundle loaded, computing hash...")
+            project_bundle = load_bundle_with_progress(bundle_dir, validate_hashes=False, console_instance=console)
+            console.print("[dim]Computing hash...[/dim]")
 
             summary = project_bundle.compute_summary(include_hash=True)
             project_hash = summary.content_hash
@@ -327,54 +318,69 @@ def enforce_sdd(
                 console.print(f"[dim]Frozen sections: {len(sdd_manifest.frozen_sections)}[/dim]")
                 # TODO: Implement hash-based frozen section validation in Phase 6
 
-            # 4. Validate OpenAPI/AsyncAPI specs with Specmatic (if found)
-            console.print("\n[cyan]Validating API specifications...[/cyan]")
+            # 4. Validate OpenAPI/AsyncAPI contracts referenced in bundle with Specmatic
+            console.print("\n[cyan]Validating API contracts with Specmatic...[/cyan]")
             import asyncio
 
             from specfact_cli.integrations.specmatic import check_specmatic_available, validate_spec_with_specmatic
 
-            base_path = Path(".")
-            spec_files = []
-            for pattern in [
-                "**/openapi.yaml",
-                "**/openapi.yml",
-                "**/openapi.json",
-                "**/asyncapi.yaml",
-                "**/asyncapi.yml",
-                "**/asyncapi.json",
-            ]:
-                spec_files.extend(base_path.glob(pattern))
+            is_available, error_msg = check_specmatic_available()
+            if not is_available:
+                console.print(f"[dim]💡 Tip: Install Specmatic to validate API contracts: {error_msg}[/dim]")
+            else:
+                # Validate contracts referenced in bundle features
+                # PlanBundle.features is a list, not a dict
+                contract_files = []
+                features_iter = (
+                    plan_bundle.features.values() if isinstance(plan_bundle.features, dict) else plan_bundle.features
+                )
+                for feature in features_iter:
+                    if feature.contract:
+                        contract_path = bundle_dir / feature.contract
+                        if contract_path.exists():
+                            contract_files.append((contract_path, feature.key))
 
-            if spec_files:
-                console.print(f"[dim]Found {len(spec_files)} API specification file(s)[/dim]")
-                is_available, error_msg = check_specmatic_available()
-                if is_available:
-                    for spec_file in spec_files[:5]:  # Validate up to 5 specs
-                        console.print(f"[dim]Validating {spec_file.relative_to(base_path)} with Specmatic...[/dim]")
+                if contract_files:
+                    console.print(f"[dim]Found {len(contract_files)} contract(s) referenced in bundle[/dim]")
+                    for contract_path, feature_key in contract_files[:5]:  # Validate up to 5 contracts
+                        console.print(
+                            f"[dim]Validating {contract_path.relative_to(bundle_dir)} (from {feature_key})...[/dim]"
+                        )
                         try:
-                            result = asyncio.run(validate_spec_with_specmatic(spec_file))
+                            result = asyncio.run(validate_spec_with_specmatic(contract_path))
                             if not result.is_valid:
                                 deviation = Deviation(
                                     type=DeviationType.CONTRACT_VIOLATION,
                                     severity=DeviationSeverity.MEDIUM,
-                                    description=f"API specification validation failed: {spec_file.name}",
-                                    location=str(spec_file),
-                                    fix_hint=f"Run 'specfact spec validate {spec_file}' to see detailed errors",
+                                    description=f"API contract validation failed: {contract_path.name} (feature: {feature_key})",
+                                    location=str(contract_path),
+                                    fix_hint=f"Run 'specfact spec validate {contract_path}' to see detailed errors",
                                 )
                                 report.add_deviation(deviation)
-                                console.print(f"  [bold yellow]⚠[/bold yellow] {spec_file.name} has validation issues")
+                                console.print(
+                                    f"  [bold yellow]⚠[/bold yellow] {contract_path.name} has validation issues"
+                                )
+                                if result.errors:
+                                    for error in result.errors[:2]:
+                                        console.print(f"    - {error}")
                             else:
-                                console.print(f"  [bold green]✓[/bold green] {spec_file.name} is valid")
+                                console.print(f"  [bold green]✓[/bold green] {contract_path.name} is valid")
                         except Exception as e:
                             console.print(f"  [bold yellow]⚠[/bold yellow] Validation error: {e!s}")
-                    if len(spec_files) > 5:
+                            deviation = Deviation(
+                                type=DeviationType.CONTRACT_VIOLATION,
+                                severity=DeviationSeverity.LOW,
+                                description=f"API contract validation error: {contract_path.name} - {e!s}",
+                                location=str(contract_path),
+                                fix_hint=f"Run 'specfact spec validate {contract_path}' to diagnose",
+                            )
+                            report.add_deviation(deviation)
+                    if len(contract_files) > 5:
                         console.print(
-                            f"[dim]... and {len(spec_files) - 5} more spec file(s) (run 'specfact spec validate' to validate all)[/dim]"
+                            f"[dim]... and {len(contract_files) - 5} more contract(s) (run 'specfact spec validate' to validate all)[/dim]"
                         )
                 else:
-                    console.print(f"[dim]💡 Tip: Install Specmatic to validate API specs: {error_msg}[/dim]")
-            else:
-                console.print("[dim]No API specification files found[/dim]")
+                    console.print("[dim]No API contracts found in bundle[/dim]")
 
             # Generate output report
             output_format_str = output_format.lower()
